import { Client } from "pg";
import { Dialect, Sequelize } from "sequelize";
import envVars from "../config";
import { initBoat } from "./models/Boat";
import { initBoatType } from "./models/BoatType";
import { initCheckIn } from "./models/CheckIn";
import { initEmployee } from "./models/Employee";
import { initBoatHasSport } from "./models/BoatHasSport";
import { initSport } from "./models/Sport";

const initializeDatabase = async () => {
  const client = new Client({
    host: envVars.DB_HOST,
    user: envVars.DB_USER,
    password: envVars.DB_PASSWORD,
  });

  // https://stackoverflow.com/questions/18389124/simulate-create-database-if-not-exists-for-postgresql
  await client.connect();
  console.log(`DB Name: ${envVars.DB_NAME}`);
  const isDatabaseExistingQuery = await client.query(
    `SELECT * FROM pg_database WHERE datname='${envVars.DB_NAME}'`
  );
  // console.log(isDatabaseExistingQuery);
  const isDatabaseExisting = isDatabaseExistingQuery.rowCount === 1;
  if (!isDatabaseExisting) {
    await client.query(`CREATE DATABASE "${envVars.DB_NAME}"`);
  }
  client.end();

  const sequelize = new Sequelize(
    envVars.DB_NAME,
    envVars.DB_USER,
    envVars.DB_PASSWORD,
    {
      host: envVars.DB_HOST,
      dialect: envVars.DB_DRIVER as Dialect,
      logging: false,
    }
  );

  await initBoat(sequelize);
  await initBoatType(sequelize);
  await initCheckIn(sequelize);
  await initEmployee(sequelize);
  await initBoatHasSport(sequelize);
  await initSport(sequelize);

  await sequelize.sync();
  return sequelize;
};

export default initializeDatabase;
